#include <avr/io.h>
#include <avr/interrupt.h>

#include "taster.h"
#include "eiuhr.h"
#include "ports.h"
#include "countdown.h"

uint16_t set_n_bits (uint8_t);

uint8_t is_idle;

enum 
{
	NO_KEY = 0,
	
	KEY_EINER   = 1,
	KEY_ZEHNER  = 2,
	
	KEY_EINER_L  = KEY_EINER  + KEY_LONG,
	KEY_ZEHNER_L = KEY_ZEHNER + KEY_LONG
};


taste_t tasten[] = 
{
      [0] = { .key = KEY_EINER,  .mode = KM_LONG }
    , [1] = { .key = KEY_ZEHNER, .mode = KM_LONG }
};

ISR (SIG_TIMEBASE)
{
    static uint8_t interrupt_num_10ms;
    static uint8_t kathode;
    
    eiuhr_aus();
    
    uint8_t irq_num = interrupt_num_10ms;
   
    // interrupt_num_10ms erhhen und mit Maximalwert vergleichen
    if (++irq_num == IRQS_PER_10MS)
    {
        // 10 Millisekunden sind vorbei
        // interrupt_num_10ms zurcksetzen
        irq_num = 0;
    }

    interrupt_num_10ms = irq_num;
    
    if (irq_num == 0)
    {
        // Die Entprellung auf lngere Intervalle
        // als auf irq_num=? basierend. Ansonsten prellte
        // der 10-er Taster hin und wieder.
        static uint8_t tast_num;
        if (tast_num & 1)
            get_taster (& tasten[0], IS_SET (PORT_EINER));
        else
            get_taster (& tasten[1], IS_SET (PORT_ZEHNER));
        tast_num++;
    }
    if (irq_num == 1)		eiuhr_beep();
    if (irq_num == 2)		job_countdown();
    
    if (is_idle)
        return;
        
    uint8_t k = kathode;
    
    // k = 0,1,2,0,1,2,0,...
    if (++k == 3)
        k = 0;
    
    if (!eiuhr.beeps)    
        eiuhr_set_leds (kathode = k);
}

uint16_t set_n_bits (uint8_t n)
{
	return (1 << n) - 1;
}

int main (void)
{
	uint8_t run = RUN_IDLE;
    
	// Disable AnalogComparator
	ACSR = 1 << ACD;
	
    eiuhr.hell = START_HELL;
    
	goto _sleep;
	
	while (1)
	{
		if (run == RUN_IDLE)
		{
			if (IS_SET (PORT_EINER) && IS_SET (PORT_ZEHNER))
            {
_sleep:	        standby();
                is_idle = 0;
                
                cli();
                taster = 0;
                tasten[0].delay = 0;
                tasten[1].delay = 0;
                tasten[0].old = 0;
                tasten[1].old = 0;
                
                eiuhr.phase = eiuhr.beeps = 0;
                sei();
            }
			else
				run = RUN_AWAIT_KEY;
		}
			
		// I just was to lazy to figure out what must be volatile
		// just clobber memory and everything is fine
		asm volatile ("; BARRIER" ::: "memory");

		uint8_t tast = taster;
		
		uint8_t zeit0 = count.min.eiuhr;
		uint8_t zeit_einer  = zeit0;
		uint8_t zeit_zehner = 0;
        
		// Einer  = zeit0 mod 10
		// Zehner = zeit0 div 10
		while (zeit_einer >= 10)
		{
			zeit_einer  -= 10;
			zeit_zehner +=  1;
		}
        
        // 1 Stunde = 60 Minuten
        if (zeit_zehner >= 6)
            zeit_zehner -= 6;
            
        //////////////////////////////////////////////////////    

        // + 1 Minute mit berlauf
		if (tast == KEY_EINER)
			if (++zeit_einer >= 10)
				zeit_einer = 0;
		
        // Genau 5 Minuten, 
        // ausser bei Minute #59 --> Helligkeit wechseln
		if (tast == KEY_EINER_L)
		{
			if (zeit0 == 59)
				eiuhr.hell = !eiuhr.hell;
			else
			{
				zeit_einer  = ZEIT_EINER;
				zeit_zehner = 0;
			}
		}

        // + 1 Zehner mit berlauf    
		if (tast == KEY_ZEHNER)
		{
			if (++zeit_zehner >= 6)
				zeit_zehner = 0;
		}
		else if (tast == KEY_ZEHNER_L)
        // Abbruch
		{
			// force RUN_IDLE below
			zeit0 = 1;
			zeit_einer = zeit_zehner = 0;
            
            is_idle = 1;
            eiuhr.beeps = 0;
            eiuhr.phase = 0;
        }
        
		if (tast)
		{
            // Tastaturpuffer leeren
			taster = NO_KEY;	
			
            // #Minuten fr Countdown berechnen
            uint8_t min = zeit_einer + 10 * zeit_zehner;

			// refill the current minute to 60 seconds
			count.sec.timer_1m = 60;

			// set the very countdown time	
			count.min.eiuhr = min;
			
			run = RUN_DOWN;
			
			// countdown aborted --> go asleep
			if (is_idle || (min != zeit0 && min == 0))
            {
                // Taster hart entprellen!!!
                const uint8_t debounce = 4;
            
                count.ms10.debounce = debounce;
                while (count.ms10.debounce)
                {
                    if (!IS_SET (PORT_ZEHNER) || !IS_SET (PORT_EINER))
                        count.ms10.debounce = debounce;
                }

				run = RUN_IDLE;
            }    
		}

		// # leuchtende LEDs an Display-Routine bergeben
		eiuhr.bits_einer = set_n_bits (zeit_einer);
        
        uint8_t bits_zehner = set_n_bits (zeit_zehner);
		eiuhr.bits_zehner = bits_zehner;
        
		// countdown reaches 0 --> beep BEEP_GROUPS times
		if (!count.min.eiuhr && run == RUN_DOWN)
			run = RUN_BEEP, eiuhr.beeps = BEEP_GROUPS;
		
		// done the beeps --> go asleep
		if (!eiuhr.beeps && run == RUN_BEEP)
			run = RUN_IDLE;	
	}
}
