#ifndef TASTER_H
#define TASTER_H

#include <stdint.h>

// `taster' bekannt machen
// Wurde keine Taste gedrckt, dann enthlt diese Variable den Wert 0.
// Wirde eine Taste gedrckt, enthlt diese Variable die Komponente
// .key des gedrckten Tasters (bzw. den um KEY_LONG vergrerten
// von .key, wenn es sich um einen langen Tastendruck handelt).
// Es werden nur dann neue Tastendrucke angenommen, wenn taster=0 ist.
// Hat man in der Anwandung alse ein Taster-Ereignis verarbeitet,
// muss man taster=0 setzen. `taster' berhlt sich also wie ein
// Tastaturpuffer der Lnge 1.

extern volatile uint8_t taster;

// Ein Taster
typedef struct 
{
    // Betriebsmodus
    // KM_SHORT   einfach nur Tastendruck melden
    // KM_LONG    Tastendruck mit Unterscheidung lang/kurz
    // KM_REPEAT  bleibt die Taste gedrckt, so wird immer wieder der gleiche
    //            Taster-Wert geliefert, hnlich wie bei einer PC-Tastatur
	// KM_SKIP    Verwendet in get_taster() den vormaligen Wert von 'tast' fr
    //            den Fall, da momentan kein Wert verfgbar ist
	uint8_t mode;
	
	// Wird der Taster gedrckt, wird dieser Wert in `taster' geschrieben.
	// bzw. bei einem langen Tastendruck .key + KEY_LONG 
	// Dieser Wert muss ungleich Null sein.
	uint8_t key;

	// Wird intern gebraucht	
	uint8_t delay;
	uint8_t old;
} taste_t;

// Alle 10ms (oder so) aufrufen
extern void get_taster (taste_t * const, uint8_t);

enum
{
	KM_SHORT,
	KM_LONG,
	KM_REPEAT,
	KM_SKIP
};

// Zeiten (in Einheiten von 10 Millisekunden) fr:
// Anfangsverzgerung bis Auto-Repeate loslegt (.mode = KM_REPEAT)
#define KEYREPEAT_DELAY 50
// Zeitverzgerung zwischen zwei Auto-Repeates (.mode = KM_REPEAT)
#define KEYREPEAT       30
// Ab dieser Dauer ist ein Tastendruck nicht mehr "kurz",
// sondern "lang" (.mode = KM_LONG)
#define KEYDELAY_LONG   30

// Wird auf .key draufaddiert, wenn es sich bei (.mode = KM_LONG)
// um einen langen Tastendruck handelt.
// Bei mehr als 4 Tastern muss dieser Wert vergrert werden!
#define KEY_LONG 4

#endif /* TASTER_H */
